/* moonsun.h    Header file for moon and sun computations.
 * Initial write  29 Mar 99   Bob Larkin  W7PUA
 * Made documentation somewhat legible, 8 Feb 01  Bob Larkin
 *
 * The associated programs are based on a variety of sources including
 * the Astronomical Almanac and its background book. Also the Naval
 * Observatory web site and discussions of time scales. Major credit must
 * go to the several books for there algorithmic contributions:
 *  Peter Duffett-Smith, "Astronomy with your Personal Computer," Second
 *        edition, Cambridge University Press, 1990.
 *  Jean Meeus, "Astronomical Formulae for Calculators," Willman-Bell, Inc,.
 *        1982.
 *  Oliver Montenbruck, "Practical Ephemeris Calculations," Springer-Verlag,
 *        1987.
 *  U.S. Naval Observatory, "Multiyear Interactive Computer Almanac,"
 *        Willman-Bell, Inc., 1998.
 */

#define SOL     299792458.0		/* Speed of light in vacuum, m/s */
#define AU      149597870150.0		/* An au, in meters */
#define NM      1852.0			/* Meters */
#define REARTH  6378150.0		/* m, at equator; Fischer-1968 (NA) */
#define REARTHP 6356768.337		/*  & at poles,   e=1/298.3 */
#define PI      3.14159265358979
#define TPI     6.28318530717959
#define PION2   1.57079632679490
#define KRD     0.0174532925199433
#define KDR     57.2957795130823
#define KHR     3.81971863420549
#define KRH     0.261799387799149
#define REM_MEAN 382830000.0

#define d2r(x)	(KRD*(x))
#define r2d(x)	(KDR*(x))
#define h2r(x)	(KRH*(x))
#define r2h(x)	(KHR*(x))
#define round(x) ((int)floor((x)+0.5))
/* fract_part(x) =0.7 for x=0.7, 1.7,...;  also=0.3 for x=-0.7, -1.7,... */
#define fract_part(x)	((x)-floor(x))
#define fsign(x) ((x)/fabs(x))		/* +1 or -1 */

/* The next synonym is due to the formulas being reversible */
#define cnvra2ha cnvha2ra

/* The various descriptors of an angle as degrees, minutes and seconds are
 * carried in the following structure.
 */
struct dms
	{
	double   sec;
	int      min;
	int      deg;
	int      sign;
	char     asc[17];
	};

/* This structure carries time when described as hours, minutes and seconds.
 */
struct hms
	{
	double   sec;
	int      min;
	int      hr;
	int      sign;
	char     asc[16];
	};

/* Some envionmental quantities change slowly and need be calculated only
 * once per day.  These are gathered here as Earth enviornment structure.
 * Note dj & jd are for integral days only and do not include local time.
 */
struct eenv
	{
	double	day;
	int	month;
	int	year;
	double	dj;		/* Julian days since 0.5 Jan 1900 */
	double	jd;		/* Julian days,  dj+2415020.0 */
	double	dp;		/* Nutation in longitude, radians */
	double	ddo;		/* Nutation in obliquity, radians */
	double	ob;		/* Obliquity of the ecliptic, radians */
	};

/* Each observer location is carried by this structure:
 */
struct location
	{
	double   lat;		/* In radians (-PION2, PION2)  */
	double   lon;		/* Radians (-PI, PI),  East is +   */
	double   ht;		/* Height in meters */
	int	 time_zone;	/* West of Greenwich is neg */
	int	 dst;		/* 0=no DST, Hrs ahead of zone time */
	char	 nm[7];		/* Short name, 6 char max */
	char	 name[32];	/* Detailed name or info */
	};

/* A pair of angles, az, el.  */
struct azel
	{
	double az;		/* Radians */
	double el;		/* Radians */
	};

/* A pair of angles, lat, lon.  Use structure location to describe a
 * place on Earth, not latlon
 */
struct latlon
	{
	double lat;		/* Radians */
	double lon;		/* Radians */
	};

/* A pair of angles, ra, dec.  */
struct radec
	{
	double ra;		/* Radians */
	double dec;		/* Radians */
	};
/* A pair of angles, ha, dec.  */
struct hadec
	{
	double ha;		/* Radians */
	double dec;		/* Radians */
	};

/* Function declarations:  */
void		cnvdd2dms(double dd, struct dms *padms);
void		cnvdh2hms(double dh, struct hms *pthms);
double 		cnvdms2dd(struct dms *padms);
double		cnvhms2dh(struct hms *pthms);
int		julday(struct eenv *earth_enviornment);
double		cnvlt2gst(double time, struct eenv *e,
		    struct location *ploc, double *put);
void		cnvut2tdt(time_t *put, struct tm *ptdt);
void		cnvhadec2azel
		    (struct hadec *pi, struct location *pl, struct azel *pae);
void		cnvazel2hadec
		    (struct azel *pi, struct location *pl, struct hadec *po);
double		cnvha2ra(double ha, double gst, double lon);
void		obliq(struct eenv *earth_enviornment);
void		nutat(struct eenv *earth_enviornment);
void		cnvll2radec
		    (struct latlon *pi, struct radec *po,
		     struct eenv *earth_enviornment);
void		parallax
		    (double lat, struct hadec *phdt,
		     double hp, struct hadec *phda);
int 		anomaly(double am, double ec, double *at, double *ae);
void		sun(double dj, double ut, double *psl, double *psed);
void		moon(double dj, double *lon, double *lat, double *ppm);
double		doppler(struct tm *ptm,
                        struct azel *pae,
                        double *rem,
		        double fhz,
                        struct location *plc);
